% Face image denoising
% noise level: sigma = 12.8
% 1 test face image
% 5 tests

% Stage 1: Learn Omega
% co-sparsity: l = [40 80]
% training data: Type I, clean patches extracted from other face images
% ADL Algorithms:  (1)ASimCO; (2) IN-ASimCO(mu=0.2)

% Stage 2: Image Recovery
% Lambda = {0.5 0.3 0.1 0.05 0.01 0.002};


l_vec = [40 80];
sigma=12.8;

for num_l = 1:length(l_vec)
    l = l_vec(num_l);
    % load clean image
    A = double(imread('yaleB15_P00A-005E-10.pgm')); 
    n = 8;
    d = n^2;
    
    for num_test = 1:5
        % generate noisy image
        AN = double(A) + double(sigma*randn(size(A))); % jing - noisy image AN
        PSNRnoisy(num_test) = 10*log10(255.^2/mean((AN(:)-double(A(:))).^2));
        
        %% Learn Omega=================
        M = d; % Signal Dimensiona (8 by 8 image patches)
        N = 2*M; % Analysis Space Dimension
        L = 128*N; % Number of Training Samples
        
        % load training data
        load(['ParamFaceYale',num2str(M),'X',num2str(N),'X',num2str(L),'.mat'])
        
        Xntrain = Y;
        Xntrain = double(Xntrain) - ones(M,1)*mean(Xntrain);
        
        param.initialDictionary = normr(randn(N,M));
        param.itN =1000;
        param.cosparsity = l;
        param.numOmegaUpdate = 1;
        
        % Algorithm 1: ASimCO        
        disp('ASimCO');
        [Omega] = analysis_SimCO_normalizedRowOmega(Xntrain, param);
        Omega_all(:, :, 1) = Omega;
        
        % Algorithm 2: IN-ASimCO, mu_0 = 0.2
        disp('IN_ASimO: mu=0.2');
        param.mu = 0.2;
        [Omega] = In_AnalysisSimCO(Xntrain, param);
        Omega_all(:, :, 2) = Omega;        
        % =================
        
        %% Recover Image
        Lambda = {0.5, 0.3, 0.1, 0.05, 0.01, 0.002}; % Lagrange Multiplier in the Objective Function
        for num_lambda = 1:length(Lambda)
            lambda = Lambda{num_lambda};
            for num_alg = [1 2]  % 4 Algorithms
                Omega = Omega_all(:, :, num_alg);
                
                % reshape image In - overlapping patches
                Y = im2col(AN, [n, n], 'sliding');
                
                Ym = mean(Y);
                Y = double(Y) - ones(d,1)*Ym;
                
                % Using a learned operator
                [X,err] = AnalysisL1Denoising(Y,Omega,lambda);
                Xdn = X;
                % Add DC
                Xdn = Xdn + repmat(Ym,d,1);
                
                [MM,NN]=size(AN);
                Idn=col2imstep(Xdn,[MM NN],[n n]);
                cnt=countcover([MM NN],[n n],[1 1]);
                Idn=Idn./cnt;
                Idn=max(0,min(255,Idn));
                
                PSNRdn = 10*log10(255.^2/mean((Idn(:)-double(A(:))).^2));
                
                PSNRdn_all(1, num_lambda, num_alg, num_test) = PSNRdn;
            end
        end
    end
    
    % averaged value of 5 tests
    PSNRdn_ASimCO          = mean(PSNRdn_all(1, :, 1, :), 4);
    PSNRdn_INASimCO_02 = mean(PSNRdn_all(1, :, 2, :), 4);
    
    filename = ['PSNRdnFace_cleanOmega_ASimCO_INASimCO_sigma_12_8_l', num2str(l), '_5tests'];
    save(filename);
end

