% Face image denoising
% noise level: sigma = 45
% 1 test face image
% 5 tests

% Stage 1: Learn Omega
% co-sparsity: l = [40 80]
% training data: Type II, patches extracted from the noisy face image
% ADL Algorithms:  (1)ASimCO; (2) IN-ASimCO(mu=0.2)

% Stage 2: Image Recovery
% Lambda = {0.5 0.3 0.1 0.05 0.01 0.002};

l_vec = [40 80];
sigma =45;

for num_l = 1:length(l_vec)
    l = l_vec(num_l);
    % for different l, run 5 tests
    for testNo = 1:5
        % generate noisy image
        A = double(imread('yaleB15_P00A-005E-10.pgm')); % jing - clean image A
        AN = double(A) + sigma*double(randn(size(A))); % jing - noisy image AN
        PSNRnoisy = 10*log10(255.^2/mean((AN(:)-double(A(:))).^2));
        
        %% Learn Omega ============
        n = 8;
        d = n^2;
        M = d;
        N = 2*M;
        
        Ms = sqrt(M);
        X = im2col(AN, [Ms, Ms], 'sliding');
        num_patches = 16384;
        IdxN=randperm(size(X, 2));
        pos=IdxN(IdxN(1:min(num_patches,size(X, 2))));
        Xntrain=X(:,pos);
        Xntrain = double(Xntrain) - ones(M,1)*mean(Xntrain);        
        
        param.initialDictionary = normr(randn(N,M));
        param.itN =1000;
        param.cosparsity = l;
        param.numOmegaUpdate = 1;
        
        % Algorithm 1: ASimCO + decorrelation step in Analysis K-SVD (mu=0.2)
        disp('ASimCO + decorrelation step in Analysis K-SVD');
        param.mu = 0.2;
        [Omega] = ASimCO_RandomVectorDecor(Xntrain, param);
        Omega_all(:, :, 1) = Omega;
        
        % Algorithm 2: ASimCO + decorrelation step in IK-SVD
        disp('ASimCO + decorrelation step in IK-SVD');
        [Omega] = ASimCO_IKSVD_Decor(Xntrain, param);
        Omega_all(:, :, 2) = Omega;     
        % =======================
        
        %% Recover Image
        Lambda = {0.5 0.3 0.1 0.05 0.01  0.002}; % Lagrange Multiplier in the Objective Function
                % for each l, test differnt lambda
        for num_lambda = 1:length(Lambda)
            lambda = Lambda{num_lambda};
            for num_alg = [1 2] % 2 Algorithms
                Omega = Omega_all(:, :, num_alg);
                
                Y = im2col(AN, [n, n], 'sliding');
                
                Ym = mean(Y);
                Y = double(Y) - ones(d,1)*Ym;
                
                % Using a learned operator
                [Xdn,err] = AnalysisL1Denoising(Y,Omega,lambda);
                % Add DC
                Xdn = Xdn + repmat(Ym,d,1);
                
                [MM,NN]=size(AN);
                Idn=col2imstep(Xdn,[MM NN],[n n]);
                cnt=countcover([MM NN],[n n],[1 1]);
                Idn=Idn./cnt;
                Idn=max(0,min(255,Idn));
                
                PSNRdn = 10*log10(255.^2/mean((Idn(:)-double(A(:))).^2));
                
                PSNRdn_all(1, num_lambda, num_alg, testNo) = PSNRdn;
            end
        end
    end
    % save a file for each l
    % averaged value of 5 tests
    PSNRdn_ASimCO_Random_mean          = mean(PSNRdn_all(1, :, 1, :), 4);
    PSNRdn_ASimCO_IKSVD_mean = mean(PSNRdn_all(1, :, 2, :), 4);
    
    filename = ['PSNRdnFace_noisyOmega_ASimCO_Random_IKSVD_sigma45_l', num2str(l), '_5tests'];
    save(filename);
end


