% Natural image denoising
% noise level: sigma = 45
% 5 test images: Boats, House, Lena, Pepper
% 5 tests

% Stage 1: Learn Omega
% co-sparsity: l = [40 80]
% training data: Type II, patches extracted the noisy image
% ADL Algorithms:  (1)ASimCO-Random; (2) IKSVD
% Stage 2: Image Recovery
% Lambda = {0.5 0.3 0.1 0.05 0.01 0.002};

sigma = 45;
l_vec = [40 80];
ImgName_cell = {'boats.png', 'house.png', 'lena256.bmp', 'peppers256.png'};

for num_img = 1:size(ImgName_cell,2)
    % load clean image
    ImgName = ImgName_cell{num_img};
    A = imread(ImgName);
    if (strcmp(ImgName, 'boats.png'))
        A = imresize(A, 0.5);
    end
    A = double(A);
    
    for num_l = 1:length(l_vec)
        l = l_vec(num_l);
        
        % for different l, run 5 tests
        for testNo = 1:5
            % generate noisy image
            AN = double(A) + sigma*randn(size(A));
            PSNRnoisy = 10*log10(255.^2/mean((AN(:)-double(A(:))).^2));
            
            %% Learn Omega==================
            n = 8;
            d = n^2;
            X = im2col(AN, [n, n], 'sliding');
            % select trainign patches
            N = 20000;
            IdxN=randperm(size(X, 2));
            pos=IdxN(IdxN(1:min(N,size(X, 2))));
            Xntrain=X(:,pos);
            % remove DC
            Xntrain = double(Xntrain) - ones(d,1)*mean(Xntrain);
            
            p = 2*d;
            OmegaInit = normr(randn(p, d));
            
            param.initialDictionary = OmegaInit;
%             param.itN =2000;
            param.itN =2;
            param.cosparsity = l;
            param.numOmegaUpdate = 1;
            
            % Algorithm 1: ASimCO + decorrelation step in Analysis K-SVD (mu=0.2)
            disp('ASimCO + decorrelation step in Analysis K-SVD');
            param.mu = 0.2;
            [Omega] = ASimCO_RandomVectorDecor(Xntrain, param);
            Omega_all(:, :, 1) = Omega;
            
            % Algorithm 2: ASimCO + decorrelation step in IK-SVD
            disp('ASimCO + decorrelation step in IK-SVD');
            [Omega] = ASimCO_IKSVD_Decor(Xntrain, param);
            Omega_all(:, :, 2) = Omega;
            % ===========================
            
            %% Recovery Image
            Lambda = {0.5 0.3 0.1 0.05 0.01 0.002};
            % for each l, test differnt lambda
            for num_lambda = 1:length(Lambda)
                lambda = Lambda{num_lambda};
                for num_alg = [1 2]
                    Omega = Omega_all(:, :, num_alg);
                    
                    % reshape image In - overlapping patches
                    Y = im2col(AN, [n, n], 'sliding');
                    
                    Ym = mean(Y);
                    Y = double(Y) - ones(d,1)*Ym;
                    
                    lambda2 = lambda;
                    % Using a learned operator
                    [Xdn,err] = AnalysisL1Denoising(Y, Omega, lambda2);
                    % Add DC
                    Xdn = Xdn + repmat(Ym,d,1);
                    
                    [MM,NN]=size(AN);
                    Idn=col2imstep(Xdn,[MM NN],[n n]);
                    cnt=countcover([MM NN],[n n],[1 1]);
                    Idn=Idn./cnt;
                    Idn=max(0,min(255,Idn));
                    
                    PSNRdn = 10*log10(255.^2/mean((Idn(:)-double(A(:))).^2));
                    
                    PSNRdn_all(1, num_lambda, num_alg, testNo) = PSNRdn;
                end
            end
        end
        
        % averaged value of 5 tests
        PSNRdn_ASimCO_Random_mean          = mean(PSNRdn_all(1, :, 1, :), 4);
        PSNRdn_IKSVD_mean = mean(PSNRdn_all(1, :, 2, :), 4);
        
        switch ImgName
            case 'boats.png'
                image = 'Boats';
            case 'house.png'
                image = 'House';
            case  'lena256.bmp'
                image = 'Lena';
            case 'peppers256.png'
                image = 'Peppers';
        end
        filename = [image, '_noisyOmega_sigma45',...
            '_ASimCO_Random_IKSVD_l', num2str(l), '_5tests'];
        save(filename);
    end
end