% Jing
% June 2, 2015 - first round revision
% Analysis SimCO + decorrelation step in Analysis K-SVD (replace correlated atoms with randomly generated-normalized atoms)
% coherence limit: mu = 0.6;
% different l, noisy case

clear

vec_l = [4 8 12 16 20 24];

ADL_AlgName = 'Analysis SimCO';
Decor_Method = 'same as Analysis K-SVD';

for num_l = 1:length(vec_l)
    l = vec_l(num_l);    
    for num_test = 1:5
        %% Generate synthetic data
        d=25; % signal dimension
        p= 50; % the number of rows of Omega
        N=50000; % the number of training samples
        
        % Generate a dictionary and normalize it
        Omega_true=normr(randn(p,d));
        
        % Generate analysis signals with co-sparsity l
        Y=zeros(d,N);
        SuppTrue=zeros(p,N);
        h=waitbar(0,'Creating signals ...');
        for i=1:N
            if rem(i,10)==0
                waitbar(i/N);
            end
            pos=randperm(p);
            SuppTrue(pos(1:l),i)=1;
            Z=null(Omega_true(pos(1:l),:));
            a=randn(size(Z,2),1);
            Y(:,i)=Z*a;
        end;
        close(h);
        Y=normc(Y);
        % add noise to training samples
        sigma=0.2/sqrt(d);
        Yn = Y+sigma*randn(size(Y));
        % generate an initial dictionary
        OmegaInit=normr(randn(p,d));
        
        %% parameters for Analysis SimCO
        Omega = OmegaInit;
        IPara.gmin = 1e-5; % the minimum value of gradient
        IPara.Lmin = 1e-6; %t4-t1 should be larger than Lmin
        IPara.t4 = 1e-2; %the initial value of t4
        IPara.rNmax = 3; %the number of iterative refinement in Part B in DictLineSearch03.m
        
        itN=2500; % the iteration number of the outer loop
        
        num_Omega_update = 1;  % the number of Omega-Update in each iteration
        
        OmegaRatio = zeros(1,itN);
        mean_cosparsity = zeros(1,itN);
        
        %% main iteration of Analysis SimCO
        for it=1:itN
            %% analysis sparse coding
            X_sparse=Omega*Yn;
            % hard thresholding
            [X_sort, index]=sort(abs(X_sparse), 1, 'ascend');
            for i=1:1:N
                X_sparse(index(1:l ,i), i)=0;
            end
            
            %% update Omega
            for i_Omega_update = 1:num_Omega_update
                [Omega, OPara]=DictLineSearch03_analysis_rowOperation(X_sparse, Omega, Yn, IPara);
            end
            
            %% decorrelation step - same as Analysis K-SVD
            mu = 0.6;
            for j = 1:size(Omega,1)
                if (any(abs(Omega(1:j-1,:)*Omega(j,:)') > mu))
                    Omega(j,:)=GenerateRandomRow(Yn); % vector generated randomly using the given training data
                    fprintf('Replaced row no. %d - already appeared\n', j);
                    Omega(j,:)=normr(Omega(j,:));
                end
            end
            % recovery rate
            [~,OmegaRatio(it)] = dictdist_normc(Omega',Omega_true');  
            fprintf('Percent recovered rows = %.2f\n',OmegaRatio(it)*100);
            
            % mean co-sparsity
            co_supp = (abs(Omega*Y)<10e-4);
            co_sparsity = sum(co_supp);
            mean_cosparsity(it) = mean(co_sparsity);
            fprintf('average cosparsity = %.3f\n',...
                mean_cosparsity(it));
        end
        
        % save the final data of this experiment
        OmegaRatio_5(num_test) = OmegaRatio(itN)
        mean_cosparsity_5(num_test) = mean_cosparsity(itN)
    end
    
    filename = ['ASimCO_RandomVectorDecor_iter2500_noisyCase_l_', int2str(l)];
    save(filename, 'OmegaRatio_5', 'mean_cosparsity_5');
    
    OmegaRatio_l(num_l) = mean(OmegaRatio_5)
    mean_cosparsity_l(num_l) = mean(mean_cosparsity_5)
end

filename = ['ASimCO_RandomVectorDecor_iter2500_noisyCase_diff_l'];
save(filename, 'vec_l', 'OmegaRatio_l', 'mean_cosparsity_l');

